//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCIAC28CARD_
   #include "pciac28card.hpp"
#endif

#ifndef _STDEXCEPT_
   #include <stdexcept>
#endif

#ifndef _IOPORT_
   #include "ioport.hpp"
#endif

#ifndef _IOTRACK_
   #include "iotrack.hpp"
#endif

#ifndef _PCIBUS_
   #include "pcibus.hpp"
#endif

#ifndef _OSIFACE_
   #include "osiface.hpp"
#endif

using namespace std;
using namespace Uniio;

//-----------------------------------------------------------------------------
PciAC28Card::PciAC28Card ( const bool inResetLevel
                         , const unsigned inCardId
                         )
  : AC28Card ( baseAddress (inCardId)
             , resetAddress (inCardId)
             , inResetLevel, "PciAC28" )
{
BFUNCTRACE_DEVELOP ();


if (inCardId >= maxNumberOfCards)
{
   throw logic_error ("Opto22PCIAC28: There can by only 32 cards in a system");
}

const PCIDeviceInfo * anInfo = PCIBus::pciBus().getDeviceInfo(0x148A, 0xAC28, inCardId );
if (!anInfo)
{
   throw runtime_error ("Opto22PCIAC28: Card not found in the system.");
} // endif

theLEDsAddress = anInfo -> baseAddress(2) & (~0x0FUL);

IOPort aResetLevelPort (resetPort()+0x0f);
IOPort aResetAssertPort (resetPort()+0x3f);

aResetLevelPort.writeChar (resetLevel() ? 0x0f : 0xff);
aResetAssertPort.writeChar (0x01);

setLEDs (LED1OnLED2On);

aResetLevelPort.writeChar (resetLevel() ? 0x0f : 0xff);
aResetAssertPort.writeChar (0x01);


OSInterface::sleep (100000); // sleep for 100 ms

setLEDs (LED1OffLED2Off);

aResetAssertPort.writeChar (0x00);

}  // end constructor


//-----------------------------------------------------------------------------
PciAC28Card::~PciAC28Card ()
{
BFUNCTRACE_DEVELOP ();
} // end destructor


//-----------------------------------------------------------------------------
PciAC28Card & PciAC28Card::setLEDs (const unsigned inLeds) 
{
BFUNCTRACE_DEVELOP ();

if ( inLeds > LED1OnLED2On )
{
   throw runtime_error ("PciAC28Card::setLEDs(): inLEDs parameter is invalid.");
} // endif

IOPort dataPort ( theLEDsAddress + inLeds );
dataPort.writeChar ( 0xFF ); // does not matter

return *this;
} // end PciAC28Card::setLEDs (..)

//-----------------------------------------------------------------------------
char PciAC28Card::readBank (const unsigned inBank) const
{
BFUNCTRACE_DEVELOP ();
IOPort aPort ( baseIOPort() + inBank*4, false);
// reading digital input bank requires 2 read operations from the same port
aPort.readChar();
return aPort.readChar();
} // end AC28Card::readBank (..)

//-----------------------------------------------------------------------------
void PciAC28Card::writeBank (const unsigned inBank, const char inData)
{
BFUNCTRACE_DEVELOP ();
IOPort aPort ( baseIOPort() + inBank*4);
aPort.writeChar(inData);
} // end AC28Card::writeBank (..)


//-----------------------------------------------------------------------------
unsigned PciAC28Card::baseAddress(const unsigned inCardId)
{
BFUNCTRACE_DEVELOP ();
if (inCardId >= maxNumberOfCards)
{
   throw logic_error ("Opto22PCIAC28: There can by only 32 cards in a system");
}

const PCIDeviceInfo * anInfo = PCIBus::pciBus().getDeviceInfo(0x148A, 0xAC28, inCardId );
if (!anInfo)
{
   throw runtime_error ("Opto22PCIAC28: Card not found in the system.");
} // endif

return (anInfo -> baseAddress(1) & (~0x0FUL));
} // end AC28Card::baseAddress (..)

//-----------------------------------------------------------------------------
unsigned PciAC28Card::resetAddress(const unsigned inCardId)
{
BFUNCTRACE_DEVELOP ();
if (inCardId >= maxNumberOfCards)
{
   throw logic_error ("Opto22PCIAC28: There can by only 32 cards in a system");
}

const PCIDeviceInfo * anInfo = PCIBus::pciBus().getDeviceInfo(0x148A, 0xAC28, inCardId );
if (!anInfo)
{
   throw runtime_error ("Opto22PCIAC28: Card not found in the system.");
} // endif

return (anInfo -> baseAddress(0) & (~0x0FUL));
} // end AC28Card::resetAddress (..)
